'use strict';

var Alias = require('../nodes/Alias.js');

var Node = require('../nodes/Node.js');

var Scalar = require('../nodes/Scalar.js');

const defaultTagPrefix = 'tag:yaml.org,2002:';

function findTagObject(value, tagName, tags) {
  if (tagName) {
    var _match$find;

    const match = tags.filter(t => t.tag === tagName);
    const tagObj = (_match$find = match.find(t => !t.format)) !== null && _match$find !== void 0 ? _match$find : match[0];
    if (!tagObj) throw new Error(`Tag ${tagName} not found`);
    return tagObj;
  }

  return tags.find(t => {
    var _t$identify;

    return ((_t$identify = t.identify) === null || _t$identify === void 0 ? void 0 : _t$identify.call(t, value)) && !t.format;
  });
}

function createNode(value, tagName, ctx) {
  var _tagName, _tagObj;

  if (Node.isDocument(value)) value = value.contents;
  if (Node.isNode(value)) return value;

  if (Node.isPair(value)) {
    var _ctx$schema$Node$MAP$, _ctx$schema$Node$MAP;

    const map = (_ctx$schema$Node$MAP$ = (_ctx$schema$Node$MAP = ctx.schema[Node.MAP]).createNode) === null || _ctx$schema$Node$MAP$ === void 0 ? void 0 : _ctx$schema$Node$MAP$.call(_ctx$schema$Node$MAP, ctx.schema, null, ctx);
    map.items.push(value);
    return map;
  }

  if (value instanceof String || value instanceof Number || value instanceof Boolean || typeof BigInt === 'function' && value instanceof BigInt // not supported everywhere
  ) {
      // https://tc39.es/ecma262/#sec-serializejsonproperty
      value = value.valueOf();
    }

  const {
    aliasDuplicateObjects,
    onAnchor,
    onTagObj,
    schema,
    sourceObjects
  } = ctx; // Detect duplicate references to the same object & use Alias nodes for all
  // after first. The `ref` wrapper allows for circular references to resolve.

  let ref = undefined;

  if (aliasDuplicateObjects && value && typeof value === 'object') {
    ref = sourceObjects.get(value);

    if (ref) {
      if (!ref.anchor) ref.anchor = onAnchor(value);
      return new Alias.Alias(ref.anchor);
    } else {
      ref = {
        anchor: null,
        node: null
      };
      sourceObjects.set(value, ref);
    }
  }

  if ((_tagName = tagName) !== null && _tagName !== void 0 && _tagName.startsWith('!!')) tagName = defaultTagPrefix + tagName.slice(2);
  let tagObj = findTagObject(value, tagName, schema.tags);

  if (!tagObj) {
    if (value && typeof value.toJSON === 'function') {
      // eslint-disable-next-line @typescript-eslint/no-unsafe-call
      value = value.toJSON();
    }

    if (!value || typeof value !== 'object') {
      const node = new Scalar.Scalar(value);
      if (ref) ref.node = node;
      return node;
    }

    tagObj = value instanceof Map ? schema[Node.MAP] : Symbol.iterator in Object(value) ? schema[Node.SEQ] : schema[Node.MAP];
  }

  if (onTagObj) {
    onTagObj(tagObj);
    delete ctx.onTagObj;
  }

  const node = (_tagObj = tagObj) !== null && _tagObj !== void 0 && _tagObj.createNode ? tagObj.createNode(ctx.schema, value, ctx) : new Scalar.Scalar(value);
  if (tagName) node.tag = tagName;
  if (ref) ref.node = node;
  return node;
}

exports.createNode = createNode;